/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGBARCODE_H_
#define FANCYDRAWINGBARCODE_H_

#include <fancydrawingglobals.h>
#include <fancyobject.h>

#include <string>

namespace Fancy {

namespace Drawing {

class Device;
class Point2;
class Color;

class FANCYDRAWING_API Barcode : virtual public Fancy::Object {

public:
    typedef Fancy::ObjPtr<Barcode> ptr_t;

    static double defaultInkReduction();

    void   setInkReduction(double r);
    double inkReduction() const;

    void   setScale(double r);
    double getScale() const;

    void   setHeight(double r);
    double getHeight() const;

    static double defaultLabelHeight();

    void   setLabelHeight(double r);
    double getLabelHeight() const;

    static double defaultBaseWidth();

    void   setBaseWidth(double r);
    double getBaseWidth() const;

    double getWidth() const;

    bool isValid() const;

    virtual std::string encoding() const;
    virtual bool checkString (const std::string &s) const;
    virtual void encode (const std::string &s);

    void draw (const Fancy::ObjPtr<Device> &dev, const Point2 &pos, HorizonalAlignment halign, VerticalAlignment valign);

protected:
    Barcode();
    virtual ~Barcode();

    struct Bar
    {
        int  space;
        int  width;
        bool extended;

        Bar() : space(1),width(1),extended(false) {}
        Bar(int s, int w, bool ex=false) : space(s),width(w),extended(ex) {}
    };

    struct Label
    {
        double xpos;
        int character;

        Label() : xpos(0),character(' ') {}
        Label(double x, int c)
        : xpos(x),character(c) {}
    };

    int n_bars;
    Bar *bars;
    int n_labels;
    Label *labels;

    void allocateEncodedString(const std::string &orig,
            int new_n_bars,
            int new_n_labels );

    void cleanup();

private:

    std::string original;

    double ink_reduction; // reduction of bar widths in device units (pt).
    double base_width; // base size of bars/spaces in device units (pt).
    double label_height; // height of labels in base size units.
    double hgt; // height of barcode in device units (pt).
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGBARCODE_H_ */
