/***********************************************************
 * $Id$
 *
 * FancyDrawing tests
 *
 * Copyright (C) 2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGCOLOR_H_
#define FANCYDRAWINGCOLOR_H_

#include <fancydrawing.h>
#include <fancyint.h>

namespace Fancy {

namespace Drawing {

class Color;

class FANCYDRAWING_API Color  {
public:

    uint8_t red;
    uint8_t green;
    uint8_t blue;
    uint8_t alpha;

    Color(): red(0),green(0),blue(0),alpha(0) {}

    Color(uint8_t _red, uint8_t _green, uint8_t _blue, uint8_t _alpha = 0xff)
    : red(_red),green(_green),blue(_blue),alpha(_alpha) {
    }

    /**
     * Construct a color from an integer in the form @c 0xaarrggbb,
     * aka @c (alpha<<24)|(red<<16)|(green<<8)|blue
     */
    Color(uint32_t intValue)
    : red((uint8_t)(intValue>>16)),green((uint8_t)(intValue>>8)),blue((uint8_t)intValue),alpha((uint8_t)(intValue>>24)) {
    }

    /**
     * @return an integer in the form @c 0xaarrggbb, aka
     *         @c (alpha<<24)|(red<<16)|(green<<8)|blue
     */
    uint32_t toInt() const {
        return ((uint32_t)blue) | (((uint32_t)green)<<8) | (((uint32_t)red)<<16) | (((uint32_t)alpha)<<24);
    }

    /**
     * @return an estimated grey value for this color, aka
     *         @c  (red*11+green*16+blue*5)/32;
     */
    uint8_t getGrey() const;

    /**
     * @return an integer in the form @c 0xrrggbb, aka
     *         @c (red<<16)|(green<<8)|blue
     */
    uint32_t toRGB() const {
        return ((uint32_t)blue) | (((uint32_t)green)<<8) | (((uint32_t)red)<<16);
    }

    /**
     * Return whether this instance is a grey, aka the red, green and blue
     * components are all equal.
     */
    bool isGrey() const {
        return this->red == this->green && this->green == this->blue;
    }

    int distance(const Color &col) const;

    bool operator==(const Color &col) const { return this->distance(col) == 0; }
    bool operator!=(const Color &col) const { return this->distance(col) != 0; }

    static const Color TRANSPARENT;
    static const Color BLACK;
    static const Color WHITE;
    static const Color RED;
    static const Color GREEN;
    static const Color BLUE;
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGIMAGE_H_ */
