/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGFONTMETRICS_H_
#define FANCYDRAWINGFONTMETRICS_H_

#include <fancydrawingglobals.h>
#include <fancyobject.h>

namespace Fancy {
namespace Drawing {

class FANCYDRAWING_API FontMetrics : public Fancy::Object {

public:
    typedef Fancy::ObjPtr<FontMetrics> ptr_t;

    class glyph_metrics_t {

        int unicode;
        double llx;
        double lly;
        double urx;
        double ury;
        double width;

    public:
        glyph_metrics_t (int uc, double _llx, double _lly, double _urx, double _ury, double _width)
        : unicode(uc),llx(_llx),lly(_lly),urx(_urx),ury(_ury),width(_width) {}

        double getLlx() const {
            return this->llx;
        }

        double getLly() const {
            return this->lly;
        }

        int getUnicode() const {
            return this->unicode;
        }

        double getUrx() const {
            return this->urx;
        }

        double getUry() const {
            return this->ury;
        }

        double getWidth() const {
            return this->width;
        }
    };

    static ptr_t getInstance(Font f);

    static std::string getGlyphName(int uc);

    static const glyph_metrics_t &getGlyphMetrics(
            const FontMetrics::ptr_t &fm, const FontMetrics::ptr_t &fallback,
            int uc, bool *was_fallback=0);

    virtual const glyph_metrics_t &getGlyphMetrics(int uc) const = 0;

    virtual double getKerning(int left, int right) const = 0;

    virtual double getAscent() const = 0;
    virtual double getAvgWidth() const = 0;
    virtual double getCapHeight() const = 0;
    virtual double getDescent() const = 0;
    virtual double getExHeight() const = 0;
    virtual Font getFont() const = 0;
    virtual double getMaxWidth() const = 0;

    virtual double getLLX() const = 0;
    virtual double getLLY() const = 0;
    virtual double getURX() const = 0;
    virtual double getURY() const = 0;

protected:
    FontMetrics();
    virtual ~FontMetrics();
};


} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGFONTMETRICS_H_ */
