/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGGLOBALS_H_
#define FANCYDRAWINGGLOBALS_H_

#include <fancydrawing.h>

namespace Fancy {
namespace Drawing {

enum PaperSize {
    A0     = 0,
    A1     = 1,
    A2     = 2,
    A3     = 3,
    A4     = 4,
    A5     = 5,
    A6     = 6
};

double paperHeight (PaperSize s) FANCYDRAWING_API;
double paperWidth  (PaperSize s) FANCYDRAWING_API;
const char* paperName (PaperSize s) FANCYDRAWING_API;

enum Unit {
    Pt     = 0,
    MM     = 1,
    CM     = 2,
    M      = 3,
    Inch   = 4
};

double ptPerUnit (Unit u) FANCYDRAWING_API;

enum  Font {
    Times_Roman                  = 0,
    Times_Bold                   = 1,
    Times_Italic                 = 2,
    Times_BoldItalic             = 3,
    Helvetica                    = 4,
    Helvetica_Bold               = 5,
    Helvetica_Oblique            = 6,
    Helvetica_BoldOblique        = 7,
    Helvetica_Narrow             = 8,
    Helvetica_Narrow_Bold        = 9,
    Helvetica_Narrow_Oblique     = 10,
    Helvetica_Narrow_BoldOblique = 11,
    Courier                      = 12,
    Courier_Bold                 = 13,
    Courier_Oblique              = 14,
    Courier_BoldOblique          = 15,
    TimesNewRoman                = 16,
    TimesNewRoman_Bold           = 17,
    TimesNewRoman_Italic         = 18,
    TimesNewRoman_BoldItalic     = 19,
    Arial                        = 20,
    Arial_Bold                   = 21,
    Arial_Italic                 = 22,
    Arial_BoldItalic             = 23,
    CourierNew                   = 24,
    CourierNew_Bold              = 25,
    CourierNew_Italic            = 26,
    CourierNew_BoldItalic        = 27,
    Verdana                      = 28,
    Verdana_Bold                 = 29,
    Verdana_Italic               = 30,
    Verdana_BoldItalic           = 31,
    ArialUnicodeMS               = 32,
    Symbol_Roman                 = 33,
    MaxTextFont                  = 32,
    MaxFont                      = 33
};

const char* fontName (Font f) FANCYDRAWING_API;

Font boldFont (Font f) FANCYDRAWING_API;
Font obliqueFont (Font f) FANCYDRAWING_API;

enum HorizonalAlignment {
    HAlignLBox      = 0, // align left on glyph bounding box
    HAlignLeft      = 1, // align left on nominal glyph width
    HAlignCenter    = 2, // align on horizontal center
    HAlignRight     = 3, // align right on nominal glyph width
    HAlignRBox      = 4  // align right on glyph bounding box
};

enum VerticalAlignment {
    VAlignTBox      = 0, // align top on glyph bounding box
    VAlignTop       = 1, // align top on nominal line height
    VAlignCenter    = 2, // align on vertical center
    VAlignBottom    = 3, // align baseline
    VAlignBBox      = 4  // align on bottom of glyph bounding box
};

enum  DashStyle {
    Solid           = 0,
    Dashed          = 1,
    Dotted          = 2,
    DashDot         = 3,
    DashDotDot      = 4,
    MaxDashStyle    = 4
};

enum CapStyle {
    FlatCap    = 0,
    RoundCap   = 1,
    SquareCap  = 2,
    MaxCapStyle = 2
};

enum JoinStyle {
    MiterJoin = 0,
    RoundJoin = 1,
    BevelJoin = 2,
    MaxJoinStyle = 2
};

const char* dashStyleName (DashStyle s) FANCYDRAWING_API;

enum TextStyle {

    /**
     * No unicode fallback, no kerning.
     */
    DefaultTextStyle       = 0,
    /**
     * No unicode fallback, enable kerning.
     */
    Kerning                = 1,
    /**
     * Enable fallback to ArialUnicodeMS for unknown characters, no kerning.
     */
    UnicodeFallback        = 2,
    /**
     * Enable fallback to ArialUnicodeMS for unknown characters, enable kerning.
     */
    UnicodeFallbackKerning = 3
};

const char* textStyleName (TextStyle s) FANCYDRAWING_API;

/**
 * @return Whether the kerning flag is set for the given text style.
 */
bool hasKerning(TextStyle textStyle) FANCYDRAWING_API;

/**
 * @return Whether the unicode fallback flag is set for the given text style.
 */
bool hasUnicodeFallback(TextStyle textStyle) FANCYDRAWING_API;

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGGLOBALS_H_ */
