/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGIMAGEBOX_H_
#define FANCYDRAWINGIMAGEBOX_H_

#include <fancydrawing.h>

#include <fancyobject.h>

#include <string>

namespace Fancy {
namespace Drawing {

class Image;

class FANCYDRAWING_API ImageBox: virtual public Fancy::Object {

private:
    int left, top;
    int width, height;

protected:
    ImageBox();
    virtual ~ImageBox();

public:
    typedef Fancy::ObjPtr<ImageBox> ptr_t;

    static ptr_t newInstance();

    static ptr_t newInstance(int left, int top, int width, int height);

    inline void setBox(int left, int top, int width, int height) {
        this->left = left;
        this->top = top;
        this->width = width;
        this->height = height;
    }

    inline int getLeft() const {
        return left;
    }

    inline int getTop() const {
        return top;
    }

    inline int getWidth() const {
        return width;
    }

    inline int getHeight() const {
        return height;
    }

    inline int getRight() const {
        return left + width - 1;
    }

    inline int getBottom() const {
        return top + height - 1;
    }

    inline void setHeight(int height) {
        this->height = height;
    }

    inline void setLeft(int left) {
        this->left = left;
    }

    inline void setTop(int top) {
        this->top = top;
    }

    inline void setWidth(int width) {
        this->width = width;
    }

    bool isValid() const;

    std::string toString() const;
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif // FANCYDRAWINGIMAGEBOX_H_
