/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGIMAGEHELPER_H_
#define FANCYDRAWINGIMAGEHELPER_H_

#include <fancydrawing.h>
#include <fancyobject.h>

#include <string>

namespace Fancy {
namespace Drawing {

class Image;
class ImageBox;

namespace Imagehelper {

/**
 * Force an image to be grey and brighten up a sub-image.
 *
 * @param image the image to consider.
 * @param maxGrey The grey value to map to white.
 * @param rect an optional rectangle to restrict the pixels considered
 *             for the brightening operation. If the rectangle is empty,
 *             no brightening occurrs, if the rectangle is invalid,
 *             the brightening applys to the whole image.
 * @return A grey 8 bpp image with the specified rectangle brigthened as specified.
 */
Fancy::ObjPtr<Image> FANCYDRAWING_API makeGreyAndBrigthen(const Fancy::ObjPtr<Image> & image,
        int maxGrey, const Fancy::ObjPtr<ImageBox> & rect = 0);

/**
 * Calculate the grey value to a given percentile.
 *
 * @param image the image to consider.
 * @param fraction The fraction of pixels, which shall be darker than
 *                 the returned grey value. A value of 0.95 calculates
 *                 the 95% percentile.
 * @param rect an optional rectangle to restrict the pixels considered.
 * @return The smallest grey value such that fraction*numPixels are
 *         darker. (in the range 0-255)
 */
int FANCYDRAWING_API calcGreyPercentile(const Fancy::ObjPtr<Image>&, double fraction,
        const Fancy::ObjPtr<ImageBox> &rect = 0);

/**
 * Cat image with range of rect.
 *
 * @param image the image to consider.
 * @param rect an rectangle to specify the range,
 *             if the range is null or the same as of the image, the same image is returned,
 *             if it's bigger an exception is thrown
 * @return A new image with range specified in rect.
 */
Fancy::ObjPtr<Image> FANCYDRAWING_API cropImage(const Fancy::ObjPtr<Image> &image,
        const Fancy::ObjPtr<ImageBox> &rect);

/**
 * Scale parts of an image to a target resolution.
 * If the image is indexed, it will be implicitly converted
 * to an unindexed version.
 *
 * @param image the image to consider.
 * @param rect an rectangle to specify the range,
 *             if the range is null or the same as of the image,
 *             the whole image is scaled,
 *             if it's bigger an exception is thrown
 * @param width The target width.
 * @param height The target height.
 * @return A new image with range scaled to the given target dimension.
 */
Fancy::ObjPtr<Image> FANCYDRAWING_API scaleImage(const Fancy::ObjPtr<Image> &image,
        const Fancy::ObjPtr<ImageBox> &rect, int width, int height);

} /* namespace Imagehelper */
} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGHELPER_H_ */
