/***********************************************************
 * $Id$
 *
 * FancyDrawing tests
 *
 * Copyright (C) 2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGIMAGEIO_H_
#define FANCYDRAWINGIMAGEIO_H_

#include <fancydrawing.h>
#include <fancyobject.h>

namespace Fancy {

class Channel;

namespace Drawing {

class Image;

class FANCYDRAWING_API ImageIO : public Fancy::Object {
public:

    typedef Fancy::ObjPtr<ImageIO> ptr_t;

    /**
     * Construct a new image IO instance for the given format.
     * Format may be "png", "jpeg", "bmp" or "membmp",
     * additional image format plugins may be provided in future.
     */
    static ptr_t newInstance(const std::string &format);

    /**
     * The format of this ImageIO instance.
     */
    virtual std::string getFormat() const = 0;

    /**
     * Set the pixel value as defined by the image format.
     */
    virtual Fancy::ObjPtr<Image> readImage(const Fancy::ObjPtr<Fancy::Channel> &channel) = 0;

    /**
     * Set the pixel value as defined by the image format.
     */
    virtual void writeImage(const Fancy::ObjPtr<Fancy::Channel> &channel, const Fancy::ObjPtr<Image> &image) = 0;

    /**
     * Read an image from a file setting the IO_FILENAME property on the returned image.
     */
    static Fancy::ObjPtr<Image> readImageFile(const Fancy::ObjPtr<ImageIO> &iio, const std::string &filename);

    /**
     * Read an image from a file setting the IO_FILENAME property on the returned image.
     * Format may be "png", "jpeg", "bmp" or "membmp",
     * additional image format plugins may be provided in future.
      */
    static Fancy::ObjPtr<Image> readImageFile(const std::string &format, const std::string &filename);

protected:
    ImageIO();
    virtual ~ImageIO();
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGIMAGEIO_H_ */
