/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWING_LOG_H
#define FANCYDRAWING_LOG_H

#include <fancydrawing.h>
#include <fancylog.h>

// facility masks:
// facility "fancydrawingbase"
#define  FANCYDRAWING_LOG_BASE            (1<<0)
// facility "fancydrawingfont"
#define  FANCYDRAWING_LOG_FONT            (1<<1)
// facility "fancydrawingpdf"
#define  FANCYDRAWING_LOG_PDF             (1<<2)
// facility "fancydrawingimage"
#define  FANCYDRAWING_LOG_IMAGE           (1<<3)
// facility "fancydrawingttf"
#define  FANCYDRAWING_LOG_TTF             (1<<4)
// facility "fancydrawingprinter"
#define  FANCYDRAWING_LOG_PRINTER         (1<<5)

namespace Fancy {
  namespace Drawing {

    extern FANCYDRAWING_API fancylog_service *log_service;

    // parse for logging cmdline args and filter out recognized options.
    void FANCYDRAWING_API fancylog_parse_arg (int *argc, char **argv);

    inline void vfancylog (int facility,
                           int priority, const char *fmt, va_list ap)
      {
        va_list aq;
        va_copy(aq,ap);
        ::vfancylog(log_service,facility,priority,fmt,aq);
        va_end(aq);
      }

#ifdef __GNUG__
  inline void fancylog (int facility,
                        int priority, const char *fmt, ...)
    __attribute__((format(printf,3,4)));
    
#endif

    inline void fancylog (int facility,
                          int priority, const char *fmt, ...)
      {
        va_list ap;
        va_start(ap, fmt);
        vfancylog(facility,priority,fmt,ap);
        va_end(ap);
      }
  }
}

#endif // ! FANCYDRAWING_LOG_H
