/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGPAGERANGE_H_
#define FANCYDRAWINGPAGERANGE_H_

#include <fancydrawing.h>
#include <fancyobject.h>

namespace Fancy {
namespace Drawing {

/**
 * A page range a selected by the user in the printer dialog.
 *
 * The pages are counted starting from 1 and the range is inclusive,
 * so that PageRange[from=1,to=2] comprises two pages.
 */
class FANCYDRAWING_API PageRange : virtual public Fancy::Object {

    int fromPage;
    int toPage;

public:
    typedef Fancy::ObjPtr<PageRange> ptr_t;

    static ptr_t newInstance();
    static ptr_t newInstance(int from, int to);

    int getFromPage() const {
        return this->fromPage;
    }

    void setFromPage(int fromPage) {
        this->fromPage = fromPage;
    }

    int getToPage() const {
        return this->toPage;
    }

    void setToPage(int toPage) {
        this->toPage = toPage;
    }

    std::string toString() const;

protected:
    PageRange();
    virtual ~PageRange();
};


} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGPAGERANGE_H_ */
