/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGPRINTERDEVICE_H_
#define FANCYDRAWINGPRINTERDEVICE_H_

#include <fancydrawingdevice.h>

namespace Fancy {

namespace Drawing {

class PrinterSetup;
class PrinterSelection;

/**
 * A device printing to a native Windows GDI-printer opened.
 *
 * A Linux implementation piping a PDF file to a LPD command
 * might also be supported in future.
 */
class FANCYDRAWING_API PrinterDevice : virtual public Device {

public:
    typedef Fancy::ObjPtr<PrinterDevice> ptr_t;

    /**
     * Construct a new instance by opening a printer dialog.
     *
     * The supplied setup is used to parameterize the dialog.
     *
     * The user selections a restored in the given selection pointer.
     *
     * Is a null pointer is returned, the user cancelled the printer
     * dialog.
     */
    static ptr_t newInstance(const Fancy::ObjPtr<PrinterSetup> &setup, const Fancy::ObjPtr<PrinterSelection> &selection);

    /**
     * Construct a new instance without a printer dialog by specifying
     * the printer name and an optional printer setup.
     */
    static ptr_t newInstance(const std::string &printerName, const Fancy::ObjPtr<PrinterSetup> &setup);

    /**
     * Start a new document with the given label.
     */
    virtual void startDocument(const std::string &label) = 0;

    /**
     * Finalize the currently assembled document
     */
    virtual void closeDocument() = 0;

protected:
    PrinterDevice();
    virtual ~PrinterDevice();
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGPRINTERDEVICE_H_ */
