/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGPRINTERSELECTION_H_
#define FANCYDRAWINGPRINTERSELECTION_H_

#include <fancydrawing.h>
#include <fancyobject.h>
#include <string>

namespace Fancy {

class List;

namespace Drawing {

class PageRange;

/**
 * Printer options as selected by the user in the printer dialog.
 *
 * @c copies is the number of copies selected by the user.
 *   If the number is 0, the user had no option to select the
 *   number of copies in the dialog.
 *
 * @c printerName is the name of the selected printer.
 *
 * @c pageRanges is a list of @c PageRange objects selected by the
 *   user.  If this list is null, the user did not select any page
 *   range and everything should be printed.
 */
class FANCYDRAWING_API PrinterSelection : virtual public Fancy::Object {

    int copies;
    std::string printerName;
    Fancy::ObjPtr<Fancy::List> pageRanges;

public:
    typedef Fancy::ObjPtr<PrinterSelection> ptr_t;

    static ptr_t newInstance();

    int getCopies() const {
        return this->copies;
    }

    void setCopies(int copies) {
        this->copies = copies;
    }

    Fancy::ObjPtr<Fancy::List> getPageRanges() const;

    void setPageRanges(const Fancy::ObjPtr<Fancy::List> &pageRanges);

    void addPageRanges(const Fancy::ObjPtr<PageRange> &pageRange);

    std::string toString() const;

    const std::string& getPrinterName() const {
        return this->printerName;
    }

    void setPrinterName(const std::string& printerName) {
        this->printerName = printerName;
    }

    // evtl. add papersize, orientation, ...


protected:
    PrinterSelection();
    virtual ~PrinterSelection();
};


} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGPRINTERSELECTION_H_ */
