/***********************************************************
 * $Id$
 *
 * FancyDrawing tests
 *
 * Copyright (C) 2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGTEXTBBOX_H_
#define FANCYDRAWINGTEXTBBOX_H_

#include <fancydrawingglobals.h>
#include <fancyobject.h>
#include <string>

namespace Fancy {

class List;

namespace Drawing {

class FontMetrics;
class Point2;

class FANCYDRAWING_API TextBBox : public Fancy::Object {
public:

    typedef Fancy::ObjPtr<TextBBox> ptr_t;

    static ptr_t newInstance(
            const Fancy::ObjPtr<FontMetrics> &f,
            double size,
            TextStyle textStyle);

    static ptr_t newInstance(
            const Fancy::ObjPtr<FontMetrics> &f,
            double size,
            TextStyle textStyle,
            int uc);

    static ptr_t newInstance(
            const Fancy::ObjPtr<FontMetrics> &f,
            double size,
            TextStyle textStyle,
            const std::string &s,
            size_t _pos=0,
            size_t _n=std::string::npos);

    static Fancy::ObjPtr<Fancy::List> fitSize(
            const std::string &txt,
            const Fancy::ObjPtr<FontMetrics> &f,
            double size,
            TextStyle textStyle,
            double width,
            size_t max_lines                       );


    virtual const Fancy::ObjPtr<FontMetrics> &getFontMetrics() const = 0;
    virtual double getPtSize() const = 0;
    virtual TextStyle getTextStyle() const = 0;

    virtual void scale (double fac) = 0;

    virtual const Point2 & getLL() const = 0;
    virtual const Point2 & getUR() const = 0;
    virtual const Point2 & getSize() const = 0;

    // returns width in ?
    virtual double getWidth() const = 0;

    // returns heigth in ?
    virtual double getHeight() const = 0;

    virtual int getFirstCode() const = 0;
    virtual int getLastCode() const = 0;
    virtual const int *beginCode() const = 0;
    virtual const int *endCode() const = 0;
    virtual int getCodeAt(size_t i) const = 0;

    /**
     * The pointer to the first kerning correction
     * as used by the PDF 'TJ' operator.
     */
    virtual const int *beginKerning() const = 0;
    /**
     * The pointer to the last kerning correction
     * as used by the PDF 'TJ' operator.
     */
    virtual const int *endKerning() const = 0;
    /**
     * A list of kerning corrections as used by the PDF 'TJ' operator.
     * kerning correction no. i is the correction between
     * character no. i and character no. i+1
     */
    virtual int getKerningAt(size_t i) const = 0;

    virtual size_t getLength() const = 0;
    virtual size_t getUtf8Length() const = 0;

    virtual void append (const TextBBox::ptr_t &b) = 0;
    virtual void append (int uc) = 0;

    virtual double getAnticipatedWidth (int uc) const = 0;

protected:
    TextBBox();
    virtual ~TextBBox();
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGTEXTBBOX_H_ */
