/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGTTFGLYPHEXTRACTOR_H_
#define FANCYDRAWINGTTFGLYPHEXTRACTOR_H_

#include <fancydrawing.h>
#include <fancyobject.h>

namespace Fancy {

class Channel;
class DataStream;

namespace Drawing {

/**
 * A TTF subsetter, which extracts a list of given glyphs.
 */
class FANCYDRAWING_API TTFGlyphExtractor : virtual public Fancy::Object {

public:
    typedef Fancy::ObjPtr<TTFGlyphExtractor> ptr_t;

    static ptr_t newInstance(const std::string &ttfFile);

    /**
     * Write the extracted font to os and return the number of bytes.
     */
    virtual uint32_t write(std::ostream &os) = 0;
    virtual void writeToUnicodeCMap(std::ostream &os,
                            const char *registry,
                            const char *ordering) = 0;
    virtual void writePdfWidths(std::ostream &os) = 0;

    virtual int findOrAddUnicode(int uc) = 0;

    virtual int getNGlyphs() const = 0;
    virtual int getUnicode(int gid) const = 0;

protected:
    TTFGlyphExtractor();
    virtual ~TTFGlyphExtractor();

};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGTTFGLYPHEXTRACTOR_H_ */
